@@ -119,6 +119,7 @@ gpm_engine_delayed_refresh (GpmEngine *engine)
 	gpm_cell_array_refresh (collection->mouse);
 	gpm_cell_array_refresh (collection->keyboard);
 	gpm_cell_array_refresh (collection->pda);
+	gpm_cell_array_refresh (collection->phone);
 	return FALSE;
 }
 
@@ -299,12 +300,20 @@ gpm_engine_get_summary (GpmEngine *engine)
 	unit = gpm_cell_array_get_unit (collection->primary);
 	accuracy = gpm_profile_get_accuracy_average (engine->priv->profile,
 						     unit->is_discharging);
-	if (accuracy < GPM_PROFILE_GOOD_TRUST && unit->is_present == TRUE) {
-		if (unit->is_discharging) {
-			tooltip = g_string_append (tooltip, _("Battery discharge profile is estimated\n"));
-			tooltip = g_string_append (tooltip, _("Policy actions will not be performed until battery profiled\n"));
-		} else {
-			tooltip = g_string_append (tooltip, _("Battery charge profile is estimated\n"));
+
+	if (unit->is_present == TRUE) {
+		if (accuracy == 0) {
+			if (unit->is_discharging) {
+				tooltip = g_string_append (tooltip, _("Battery discharge time is currently unknown\n"));
+			} else {
+				tooltip = g_string_append (tooltip, _("Battery charge time is currently unknown\n"));
+			}
+		} else if (accuracy < GPM_PROFILE_GOOD_TRUST) {
+			if (unit->is_discharging) {
+				tooltip = g_string_append (tooltip, _("Battery discharge time is estimated\n"));
+			} else {
+				tooltip = g_string_append (tooltip, _("Battery charge time is estimated\n"));
+			}
 		}
 	}
 
@@ -323,7 +332,12 @@ gpm_engine_get_summary (GpmEngine *engine)
 		tooltip = g_string_append (tooltip, part);
 	}
 	g_free (part);
-	part = gpm_cell_array_get_description (collection->ups);
+	part = gpm_cell_array_get_description (collection->phone);
+	if (part != NULL) {
+		tooltip = g_string_append (tooltip, part);
+	}
+	g_free (part);
+	part = gpm_cell_array_get_description (collection->pda);
 	if (part != NULL) {
 		tooltip = g_string_append (tooltip, part);
 	}
@@ -349,6 +363,8 @@ gpm_engine_get_icon (GpmEngine *engine)
 	GpmCellUnit *unit_ups;
 	GpmCellUnit *unit_mouse;
 	GpmCellUnit *unit_keyboard;
+	GpmCellUnit *unit_pda;
+	GpmCellUnit *unit_phone;
 	GpmEngineCollection *collection;
 	GpmWarningsState state;
 
@@ -368,6 +384,8 @@ gpm_engine_get_icon (GpmEngine *engine)
 	unit_ups = gpm_cell_array_get_unit (collection->ups);
 	unit_mouse = gpm_cell_array_get_unit (collection->mouse);
 	unit_keyboard = gpm_cell_array_get_unit (collection->keyboard);
+	unit_pda = gpm_cell_array_get_unit (collection->pda);
+	unit_phone = gpm_cell_array_get_unit (collection->phone);
 
 	/* we try CRITICAL: PRIMARY, UPS, MOUSE, KEYBOARD */
 	gpm_debug ("Trying CRITICAL icon: primary, ups, mouse, keyboard");
@@ -379,6 +397,10 @@ gpm_engine_get_icon (GpmEngine *engine)
 	if (unit_ups->is_present == TRUE && state == GPM_WARNINGS_CRITICAL) {
 		return gpm_cell_array_get_icon (collection->ups);
 	}
+	state = gpm_warnings_get_state (engine->priv->warnings, unit_phone);
+	if (unit_keyboard->is_present == TRUE && state == GPM_WARNINGS_CRITICAL) {
+		return gpm_cell_array_get_icon (collection->phone);
+	}
 	state = gpm_warnings_get_state (engine->priv->warnings, unit_mouse);
 	if (unit_mouse->is_present == TRUE && state == GPM_WARNINGS_CRITICAL) {
 		return gpm_cell_array_get_icon (collection->mouse);
@@ -387,6 +409,10 @@ gpm_engine_get_icon (GpmEngine *engine)
 	if (unit_keyboard->is_present == TRUE && state == GPM_WARNINGS_CRITICAL) {
 		return gpm_cell_array_get_icon (collection->keyboard);
 	}
+	state = gpm_warnings_get_state (engine->priv->warnings, unit_pda);
+	if (unit_keyboard->is_present == TRUE && state == GPM_WARNINGS_CRITICAL) {
+		return gpm_cell_array_get_icon (collection->pda);
+	}
 
 	if (engine->priv->icon_policy == GPM_ICON_POLICY_CRITICAL) {
 		gpm_debug ("no devices critical, so no icon will be displayed.");
@@ -896,6 +922,7 @@ gpm_engine_init (GpmEngine *engine)
 	collection->mouse = gpm_cell_array_new ();
 	collection->keyboard = gpm_cell_array_new ();
 	collection->pda = gpm_cell_array_new ();
+	collection->phone = gpm_cell_array_new ();
 	engine->priv->ac_adapter = gpm_ac_adapter_new ();
 
 	g_signal_connect (collection->primary, "collection-changed",
@@ -928,6 +955,15 @@ gpm_engine_init (GpmEngine *engine)
 	g_signal_connect (collection->mouse, "charge-critical",
 			  G_CALLBACK (gpm_cell_array_charge_critical_cb), engine);
 
+	g_signal_connect (collection->phone, "collection-changed",
+			  G_CALLBACK (gpm_cell_array_collection_changed_cb), engine);
+	g_signal_connect (collection->phone, "percent-changed",
+			  G_CALLBACK (gpm_cell_array_percent_changed_cb), engine);
+	g_signal_connect (collection->phone, "charge-low",
+			  G_CALLBACK (gpm_cell_array_charge_low_cb), engine);
+	g_signal_connect (collection->phone, "charge-critical",
+			  G_CALLBACK (gpm_cell_array_charge_critical_cb), engine);
+
 	g_signal_connect (collection->keyboard, "collection-changed",
 			  G_CALLBACK (gpm_cell_array_collection_changed_cb), engine);
 	g_signal_connect (collection->keyboard, "percent-changed",
@@ -981,6 +1017,7 @@ gpm_engine_start (GpmEngine *engine)
 	gpm_cell_array_set_type (collection->mouse, GPM_CELL_UNIT_KIND_MOUSE);
 	gpm_cell_array_set_type (collection->keyboard, GPM_CELL_UNIT_KIND_KEYBOARD);
 	gpm_cell_array_set_type (collection->pda, GPM_CELL_UNIT_KIND_PDA);
+	gpm_cell_array_set_type (collection->phone, GPM_CELL_UNIT_KIND_PHONE);
 
 	/* only show the battery prefs section if we have batteries */
 	prefs_server = gpm_prefs_server_new ();
@@ -1023,6 +1060,7 @@ gpm_engine_finalize (GObject *object)
 	g_object_unref (collection->mouse);
 	g_object_unref (collection->keyboard);
 	g_object_unref (collection->pda);
+	g_object_unref (collection->phone);
 
 	g_free (engine->priv->previous_icon);
 	g_free (engine->priv->previous_summary);
