@@ -46,7 +46,6 @@
 #include "gpm-common.h"
 #include "gpm-debug.h"
 #include "gpm-notify.h"
-#include "gpm-control.h"
 #include "gpm-cell-array.h"
 #include "gpm-cell.h"
 #include "gpm-cell-unit.h"
@@ -333,10 +332,10 @@ gpm_tray_icon_show_about_cb (GtkMenuItem *item, gpointer data)
 
 	gtk_window_set_default_icon_name (GPM_STOCK_APP_ICON);
 	gtk_show_about_dialog (NULL,
-			       "name", GPM_NAME,
 			       "version", VERSION,
 			       "copyright", "Copyright \xc2\xa9 2005-2007 Richard Hughes",
 			       "license", license_trans,
+			       "website-label", _("GNOME Power Manager Website"),
 			       "website", GPM_HOMEPAGE_URL,
 			       "comments", GPM_DESCRIPTION,
 			       "authors", authors,
@@ -560,6 +559,7 @@ gpm_tray_icon_activate_cb (GtkStatusIcon *status_icon,
 	/* add all device types to the drop down menu */
 	dev_cnt += gpm_tray_icon_add_device (icon, menu, icon->priv->collection->primary);
 	dev_cnt += gpm_tray_icon_add_device (icon, menu, icon->priv->collection->ups);
+	dev_cnt += gpm_tray_icon_add_device (icon, menu, icon->priv->collection->phone);
 	dev_cnt += gpm_tray_icon_add_device (icon, menu, icon->priv->collection->mouse);
 	dev_cnt += gpm_tray_icon_add_device (icon, menu, icon->priv->collection->keyboard);
 	dev_cnt += gpm_tray_icon_add_device (icon, menu, icon->priv->collection->pda);
